# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-08 17:56:18 +0900 (Wed, 08 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/http/common'
require 'thread'
require 'time.rb'

module VFS;  module SchemeHandler;
  module HTTP;
    class File < VFS::File
      include VFS::SchemeHandler::HTTP::Common
      # FIXME, TODO :utime, :delete, :unlink, :rename, :truncate,
      # :lstat :stat :chown(hmm?) and  write-open!
      # :umask
      # :expand_path :readlink

      return_false :chardev?, :pipe?, :sticky?, :blockdev?, :grpowned?,
        :setgid?, :executable_real?, :socket?, :owned?, :setuid?,
        :executable?, :symlink?

      attr_reader :fetch_thread
      attr_writer :size, :mtime
      private :size=, :mtime=

      def initialize(url)
	check_uri_syntax(url)
	super
	init_common_attr
	@mtime          = nil
	@ctime          = nil
	@size           = nil
	@fetch_thread   = nil
	@is_exists      = nil
      end

      def readable?
	# fixme? should not catch exception? check.
	begin
	  header
	rescue
	  return false
	end
	true
      end
      alias :readable_real? :readable?

      def writable?
	if directory?
	  capability?("MKCOL") || capability?("DELETE")
	else
	  capability? "PUT"
	end
      end
      alias :writable_real? :writable? 

      def file?
	! directory?
      end

      def directory?
	property("D:getcontenttype") =~ /httpd\/unix-directory/i ? true : false
      end

      def size
	@size = property("lp\d:getcontentlength") ||
	  header("content-length")
	if @size != nil
	  @size = @size.to_i
	else
	  wait_fetch
	end
	@size
      end

      def mtime
	@mtime = property("lp\d:getlastmodified") ||
	  header("last-modified")
	if @mtime != nil
	  @mtime = Time.parse(@mtime)
	end
	@mtime
      end
      # fixme; atime?
      alias :atime :mtime

      def ctime
	if property("lp\d:getlastmodified") != nil
	  @ctime = Time.at(property("lp\d:getlastmodified"))
	end
	@ctime
      end

      def ftype
	"file(http)"
      end

      def exists?
	return @is_exists unless @is_exists == nil
	begin
	  header
	rescue Net::ProtoFatalError # FIXME. original error class
	  return @is_exists = false
	end
	@is_exists = true
      end
      alias :exist? :exists?

      def wait_fetch
	if fetch_thread == nil
	  @fetch_thread = start_fetch
	end
	if fetch_thread.alive?
	  fetch_thread.join
	end
      end

      private
      def start_fetch
	log("Start fetching thread.")
	Thread.start {
	  http_fetch(url, :get) { |rep|
	    buf << rep.body
	  }
	  buf.close
	  if @size == nil
	    @size = buf.size
	  end
	  log("HTTP fetching thread has been finished.")
	}
      end

      def open_as_read(*args, &block)
	wait_fetch
	super
      end

      def open_as_write(*args, &block)
	super # FXIME! only fail........
      end
      
    end # VFS::SchemeHandler::Http::File
end; end; end


if __FILE__ == $0
#  f = "http://localhost/"
#  f = "http://localhost/~sugi/"
  f = "http://sugi.nemui.org/pub/prog/hns2td"
  vfs = VFS::SchemeHandler::HTTP::File.new(f)
  p vfs
  p VFS::scheme(f)
  p vfs.scheme
  p vfs.exists?
  p vfs.directory?
  p vfs.writable?
  p vfs.mtime
#  p VFS::SchemeHandler::HTTP::File.mtime(f) # something wrong; warning: method redefined
  p vfs.size
#  p VFS::SchemeHandler::HTTP::File.size(f)
  p vfs.basename
  p vfs.dirname
  io = vfs.open
  puts io.read(20)
  io.close
  vfs.open { |io| puts io.read(20) }
#  io = VFS::SchemeHandler::HTTP::File.open(f)
#  puts io.read(20)
#  io.close
#  VFS::SchemeHandler::HTTP::File.open(f){ |io| puts io.read(20) }
end
